/*----------------------------------------------------------------------*/
/* opengl.c --- routines defining graphics, whether OpenGL or X11	*/
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>

/*----------------------------------------------------------------------*/
/* Local includes							*/
/*----------------------------------------------------------------------*/

#ifdef TCL_WRAPPER
#include <tk.h>
#endif

#include "helpme.h"
#include "colordefs.h"

extern Colormap cmap;

/*----------------------------------------------------------------------*/
/* Function prototype declarations                                      */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/* Global variables---							*/
/* Maintain current linewidth, because we need to pass this to the	*/
/* rendering routines, since we're duplicating all of the X calls,	*/
/* and they don't pass linewidth to the geometry drawing routines.	*/
/*----------------------------------------------------------------------*/

GLuint	grXBases[2];

/*----------------------------------------------*/
/*----------------------------------------------*/

u_char
LoadFont(int num, Font font)
{
    grXBases[num] = glGenLists(256);
    if (grXBases[num] == 0)
       return 0;
    glXUseXFont(font, 0, 256, grXBases[num]);
}

/*----------------------------------------------*/
/* Set color in OpenGL (this should be replaced	*/
/* for speed. . .)				*/
/*----------------------------------------------*/

void
SetForeground(XColor *color)
{
   if (hms.cur_color != color) { 
      glColor3us((GLushort)color->red, (GLushort)color->green,
		(GLushort)color->blue);
      hms.cur_color = color;
   }
}

/*------------------------------------------------------*/
/* Set line attributes                                  */
/*------------------------------------------------------*/

void
SetLineAttributes(float lwidth)
{
   GLfloat glwidth = (lwidth <= 0) ? 1 : (GLfloat)lwidth;
   glLineWidth(glwidth);
   glPointSize(glwidth);

   glEnable(GL_LINE_SMOOTH);
   glDisable(GL_LINE_STIPPLE);
}

/*------------------------------------------------------*/
/* Draw a line 						*/
/*------------------------------------------------------*/

void
DrawLine(fpoint *l1, fpoint *l2)
{
   glBegin(GL_LINES);
   glVertex3f((GLfloat)l1->x, (GLfloat)l1->y, (GLfloat)l1->z);
   glVertex3f((GLfloat)l2->x, (GLfloat)l2->y, (GLfloat)l2->z);
   glEnd();
   return;
}

/*--------------------------------------------------------------*/
/* Draw multiple lines (compatibility function for XDrawLines()	*/
/* (Slight performance improvement over DrawLine by calling	*/
/* point-drawing routine only once in-between segments).	*/
/*--------------------------------------------------------------*/

void
DrawLines(fpoint *points, int npoints, u_char closed)
{
   int i;

   for (i = 0; i < npoints - 1; i++) {
      DrawLine(&points[i], &points[i + 1]);
   }
   if (closed == 1) {
      DrawLine(&points[npoints - 1], &points[0]);
   }
}

/*----------------------------------------------------------------------*/
/* Draw a filled polygon (compatibility function for XFillPolygon()	*/
/*----------------------------------------------------------------------*/

void
FillPolygon(fpoint *points, int npoints)
{
   int i;
   static GLdouble *v;

   glEnable(GL_POLYGON_SMOOTH);
   glBegin(GL_POLYGON);
   for (i = 0; i < npoints; i++)
      glVertex3f((GLfloat)points[i].x, (GLfloat)points[i].y, (GLfloat)points[i].z);
   glEnd();
   glDisable(GL_POLYGON_SMOOTH);
   DrawLines(points, npoints, 1);
}

/*----------------------------------------------------------------------*/
/* Draw a filled circle							*/
/*----------------------------------------------------------------------*/

void FillArc(fpoint p, int diameter, fpoint *circ_vec)
{
   GLushort lr, lg, lb;
   fpoint shadow_vec0[4] = {
      {-6.9,  4.0, 0.0},
      {-4.0,  6.9, 0.0},
      { 0.0,  8.0, 0.0},
      {-2.5,  6.0, 0.0}
   };
   fpoint shadow_vec1[3] = {
      { 0.0,  8.0, 0.0},
      { 2.5,  6.0, 0.0},
      {-2.5,  6.0, 0.0}
   };
   fpoint shadow_vec2[4] = {
      { 0.0,  8.0, 0.0},
      { 4.0,  6.9, 0.0},
      { 6.9,  4.0, 0.0},
      { 2.5,  6.0, 0.0}
   };
   fpoint highlight_vec[7] = {
      {-5.0, -2.0,  0.0},
      {-6.0, -3.0,  0.0},
      {-5.5, -5.0,  0.0},
      {-3.0, -6.0,  0.0},
      {-1.2, -6.0,  0.0},
      {-1.0, -5.0,  0.0},
      {-2.5, -3.0,  0.0}
   };
   fpoint zcirc[12];
   int i;
   float d = (float)diameter / 16.0;

   for (i = 0; i < 12; i++) {
      zcirc[i].x = p.x + circ_vec[i].x * d;
      zcirc[i].y = p.y + circ_vec[i].y * d;
      zcirc[i].z = p.z;
   }
   FillPolygon(zcirc, 12);

   lr = (GLushort)((float)hms.cur_color->red * 0.8);
   lg = (GLushort)((float)hms.cur_color->green * 0.8);
   lb = (GLushort)((float)hms.cur_color->blue * 0.8);
   glColor3us(lr, lg, lb);

   for (i = 0; i < 4; i++) {
      zcirc[i].x = p.x + shadow_vec0[i].x * d;
      zcirc[i].y = p.y + shadow_vec0[i].y * d;
      zcirc[i].z = p.z + 1;
   }
   FillPolygon(zcirc, 4);
   for (i = 0; i < 3; i++) {
      zcirc[i].x = p.x + shadow_vec1[i].x * d;
      zcirc[i].y = p.y + shadow_vec1[i].y * d;
      zcirc[i].z = p.z + 1;
   }
   FillPolygon(zcirc, 3);
   for (i = 0; i < 4; i++) {
      zcirc[i].x = p.x + shadow_vec2[i].x * d;
      zcirc[i].y = p.y + shadow_vec2[i].y * d;
      zcirc[i].z = p.z + 1;
   }
   FillPolygon(zcirc, 4);

   lr = (GLushort)((float)hms.cur_color->red * 0.6 + (float)65535 * 0.4);
   lg = (GLushort)((float)hms.cur_color->green * 0.6 + (float)65535 * 0.4);
   lb = (GLushort)((float)hms.cur_color->blue * 0.6 + (float)65535 * 0.4);
   glColor3us(lr, lg, lb);

   for (i = 0; i < 7; i++) {
      zcirc[i].x = p.x + highlight_vec[i].x * d;
      zcirc[i].y = p.y + highlight_vec[i].y * d;
      zcirc[i].z = p.z + 2;
   }
   FillPolygon(zcirc, 7);
}

/*----------------------------------------------------------------------*/
/* Draw a filled rectangle						*/
/*----------------------------------------------------------------------*/

void FillRectangle(fpoint p, int width, int height)
{
   fpoint points[4];
   int i;

   for (i = 0; i < 4; i++) {
      points[i] = p;
   }
   points[1].x += (float)width;
   points[2].x += (float)width;
   points[2].y += (float)height;
   points[3].y += (float)height;

   FillPolygon(points, 4);
}

/*----------------------------------------------------------------------*/
/* Draw a texture-mapped rectangle					*/
/*----------------------------------------------------------------------*/

void TextureRectangle(fpoint *points, TextureMap *texture, float tiles)
{
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, (GLuint)texture->id);
   glBegin(GL_QUADS);
   glTexCoord2f(0.0, 0.0);
   glVertex3f((GLfloat)points[0].x, (GLfloat)points[0].y, (GLfloat)points[0].z);
   glTexCoord2f(tiles, 0.0);
   glVertex3f((GLfloat)points[1].x, (GLfloat)points[1].y, (GLfloat)points[1].z);
   glTexCoord2f(tiles, tiles);
   glVertex3f((GLfloat)points[2].x, (GLfloat)points[2].y, (GLfloat)points[2].z);
   glTexCoord2f(0.0, tiles);
   glVertex3f((GLfloat)points[3].x, (GLfloat)points[3].y, (GLfloat)points[3].z);
   glEnd();
   glDisable(GL_TEXTURE_2D);
}

/*----------------------------------------------------------------------*/
/* Paste X11 text onto the OpenGL target drawable			*/
/*----------------------------------------------------------------------*/

void DrawString(fpoint p, char *text, int length, int nfont)
{
   glRasterPos3f((GLfloat)p.x, (GLfloat)p.y, (GLfloat)p.z);
   glListBase(grXBases[nfont]);
   glCallLists(length, GL_UNSIGNED_BYTE, (unsigned char *)text);
}

/*----------------------------------------------------------------------*/
