/*----------------------------------------------------------------------*/
/* graphic.c --- xcircuit routines handling rendered graphic elements	*/
/* Copyright (c) 2005  Tim Edwards, MultiGiG, Inc.			*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*	written by Tim Edwards, 7/11/05					*/
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <GL/gl.h>
#include <GL/glx.h>

#include <tk.h>

/*----------------------------------------------------------------------*/
/* Local includes							*/
/*----------------------------------------------------------------------*/

#include "helpme.h"
#include "colordefs.h"

/*----------------------------------------------------------------------*/
/* Global Variable definitions						*/
/*----------------------------------------------------------------------*/

extern Display  *dpy;

/*----------------------------------------------------------------------*/
/* Create a new texture image from a PPM file				*/
/* Return 1 on success, 0 on failure.					*/
/*----------------------------------------------------------------------*/

u_char new_texture(char *filename, TextureMap *texture)
{
    FILE *fg;
    int nr, width, height, imax, i, offset, tid;
    char id[5], c, buf[128];
    u_char b, rval = 0;
    GLubyte *local_image;

    fg = fopen(filename, "r");
    if (fg == NULL) goto load_error;

    /* This ONLY handles binary ppm files with max data = 255 */

    while (1) {
       nr = fscanf(fg, " %s", buf);
       if (nr <= 0) goto load_error;
       if (buf[0] != '#') {
	  if (sscanf(buf, "%s", id) <= 0)
	     goto load_error;
	  break;
       }
       else fgets(buf, 127, fg);
    }
    if ((nr <= 0) || strncmp(id, "P6", 2)) goto load_error;

    while (1) {
       nr = fscanf(fg, " %s", buf);
       if (nr <= 0) goto load_error;
       if (buf[0] != '#') {
	  if (sscanf(buf, "%d", &width) <= 0)
	     goto load_error;
	  break;
       }
       else fgets(buf, 127, fg);
    }
    if (width <= 0) goto load_error;

    while (1) {
       nr = fscanf(fg, " %s", buf);
       if (nr <= 0) goto load_error;
       if (buf[0] != '#') {
	  if (sscanf(buf, "%d", &height) <= 0)
	     goto load_error;
	  break;
       }
       else fgets(buf, 127, fg);
    }
    if (height <= 0) goto load_error;

    while (1) {
       nr = fscanf(fg, " %s", buf);
       if (nr <= 0) goto load_error;
       if (buf[0] != '#') {
	  if (sscanf(buf, "%d", &imax) <= 0)
	     goto load_error;
	  break;
       }
       else fgets(buf, 127, fg);
    }
    if (imax != 255) goto load_error;

    while (1) {
       fread(&c, 1, 1, fg);
       if (c == '\n') break;
       else if (c == '\0') goto load_error;
    }

    local_image = (GLubyte *)Tcl_Alloc(width * height * 3 * sizeof(GLubyte));

    /* Read the image data from the PPM file */

    offset = 0;
    for (i = 0; i < height * width * 3; i++) {
       fread(&b, 1, 1, fg);
       *(local_image + offset) = (GLubyte)b;
       offset++;
    }
    texture->width = width;
    texture->height = height;
    texture->data = local_image;
    glGenTextures(1, &tid);
    texture->id = (GLuint)tid;
    rval = 1;

load_error:
    fclose(fg);
    return rval;
}

/*----------------------------------------------------------------------*/
/* Darken a texture by the indicated amount, and save as a new texture	*/
/*----------------------------------------------------------------------*/

void shade_texture(TextureMap *old, TextureMap *new, float factor)
{
    int width, height, i, offset, tid;
    u_char b;
    GLubyte *new_image;

    height = old->height;
    width = old->width;
    new_image = (GLubyte *)Tcl_Alloc(width * height * 3 * sizeof(GLubyte));

    offset = 0;
    for (i = 0; i < height * width * 3; i++) {
       b = (u_char)(*(old->data + offset));
       b = (u_char)((float)b * factor);
       *(new_image + offset) = (GLubyte)b;
       offset++;
    }

    new->width = width;
    new->height = height;
    new->data = new_image;
    glGenTextures(1, &tid);
    new->id = (GLuint)tid;
}

/*----------------------------------------------------------------------*/
