#-------------------------------------------------------------------------
# Start of helpme GUI configuration file.
# This file is sourced by "helpme.tcl" (".wishrc")
#-------------------------------------------------------------------------
namespace path {::tcl::mathop ::tcl::mathfunc}

global HelpMeOps
set HelpMeOps(run) 0

toplevel .dialog
wm title .dialog "High Scorer"
wm group .dialog .
wm protocol .dialog WM_DELETE_WINDOW {}
wm withdraw .dialog
frame .dialog.textent
pack .dialog.textent -side top -padx 20 -pady 7 -fill x
frame .dialog.textent.title
entry .dialog.textent.txt -bg white -relief sunken -width 50
pack .dialog.textent.title -side top -fill x
pack .dialog.textent.txt -side bottom -fill x -expand true
label .dialog.textent.title.field -text "Enter your name"
pack .dialog.textent.title.field -side left
frame .dialog.bbar
pack .dialog.bbar -side bottom -padx 20 -pady 7 -fill x
button .dialog.bbar.okay -text Okay -command {.dialog.bbar.apply invoke ;\
        wm withdraw .dialog}
button .dialog.bbar.apply -text Apply -command \
	{helpme::username [.dialog.textent.txt get]}
button .dialog.bbar.cancel -text Cancel -command {wm withdraw .dialog}
bind .dialog.textent.txt <Return> {.dialog.bbar.apply invoke}
pack .dialog.bbar.okay -side left -ipadx 10
pack .dialog.bbar.apply -side left -ipadx 10
pack .dialog.bbar.cancel -side right -ipadx 10


proc helpme::animate {} {
   global HelpMeOps
   puts stdout "Running"
   set HelpMeOps(run) 1
   helpme::go
}

proc helpme::go {} {
   global HelpMeOps
   after 2 {
      update
      recalc
      redraw
      if {$HelpMeOps(run) == 1} {
         after idle helpme::go
      }
   }
}

proc helpme::stop {} {
   global HelpMeOps
   set HelpMeOps(run) 0
   puts stdout "Stopping"
}

proc helpme::pause_toggle {} {
   global HelpMeOps
   if {$HelpMeOps(run) == 0} {
      set HelpMeOps(run) 1
      helpme::go
   } else {
      set HelpMeOps(run) 0
   }
}

proc helpme::getusername {} {
}

#-------------------------------
# Main helpme drawing window
#-------------------------------

proc helpme::make_window {name} {

  global HELPME_VERSION HELPME_REVISION HelpMeOps tcl_platform

  toplevel $name
  wm title $name [string map {.helpme HelpMe} $name]
  wm group $name .

  # All the internal frames

  frame ${name}.menubar
  frame ${name}.mainarea -borderwidth 1 -bg #a07650

  grid propagate ${name} false
  # grid ${name}.menubar -sticky news -row 0 -column 0
  # grid ${name}.mainarea -sticky news -row 1 -columnspan 2
  grid ${name}.mainarea -sticky news -row 0 -column 0

  # grid rowconfigure ${name} 0 -weight 0
  # grid rowconfigure ${name} 1 -weight 1
  # grid rowconfigure ${name} 2 -weight 0

  grid rowconfigure ${name} 0 -weight 1

  # grid columnconfigure ${name} 0 -weight 0
  # grid columnconfigure ${name} 1 -weight 1

  grid columnconfigure ${name} 0 -weight 1

  set drawing ${name}.mainarea.drawing

  simple $drawing -bg white -commandproc "config window"

  # The drawing area ("grid" used in case we want to add extra sub-windows
  # on the sides. . .)

  grid $drawing -row 0 -column 0 -sticky news

  grid rowconfigure ${name}.mainarea 0 -weight 1
  grid columnconfigure ${name}.mainarea 0 -weight 1

  #-------------------------------------------------
  # Set default key bindings
  #-------------------------------------------------

  # (none yet---to be done, example below)
  bind ${drawing}  <ButtonPress-1> {launch}
  bind ${drawing}  <ButtonPress-2> {pull enable}
  bind ${drawing}  <ButtonPress-3> {rotate enable to}
  bind ${drawing}  <ButtonPress-4> {roll 10}
  bind ${drawing}  <ButtonPress-5> {roll -10}
  bind ${drawing}  <ButtonRelease-2> {pull disable}
  bind ${drawing}  <ButtonRelease-3> {rotate disable}
  bind ${drawing}  <KeyPress-space> {nextstate}
  bind ${drawing}  <KeyPress-Up> {roll forward}
  bind ${drawing}  <KeyPress-Down> {roll backward}
  bind ${drawing}  <KeyRelease-Up> {roll disable}
  bind ${drawing}  <KeyRelease-Down> {roll disable}
  bind ${drawing}  <KeyPress-Left> {rotate 10}
  bind ${drawing}  <KeyPress-Right> {rotate -10}
  bind ${drawing}  <KeyPress-b> {bomb}
  bind ${drawing}  <KeyPress-c> {build}
  bind ${drawing}  <KeyPress-q> {quit}
  bind ${drawing}  <KeyPress-p> {pause}

  #-------------------------------------------------
  # The top menu and message bar
  #-------------------------------------------------

  menubutton ${name}.menubar.filebutton -text File \
	-menu ${name}.menubar.filebutton.filemenu \
	-background beige

  grid ${name}.menubar.filebutton \
	-ipadx 10 -sticky news

  # The top message bar

  label ${name}.message -text \
	"Welcome to helpme v${HELPME_VERSION} rev ${HELPME_REVISION}" \
	-background beige -justify left -anchor w

  # grid ${name}.message -row 0 -column 1 -sticky news -ipadx 10

  #-------------------------------------------------
  # Create the menus
  #-------------------------------------------------

  set m [menu ${name}.menubar.filebutton.filemenu -tearoff 0]
  $m add command -label "Go" -command "helpme::animate"
  $m add command -label "Pause" -command "helpme::stop"
  $m add command -label "Quit" -command "quit"

  # Return the name of the drawing window
  return ${drawing}
}

#----------------------------------------------------------------------
# Tag callback routines
#----------------------------------------------------------------------

# (none yet)

#-----------------------------------------------------------------
# Final setup stuff before exiting back to interpreter
#-----------------------------------------------------------------

# This gets rid of the original "wish", in favor of our own window
# (or the helpmeexec, if not called from the tkcon console)

if {[string range [wm title .] 0 3] == "wish"} {
   wm withdraw .
}

if {[string range [wm title .] 0 9] == "helpmeexec"} {
   wm withdraw .
}

#----------------------------------------------------------------------
# Tag callbacks
#----------------------------------------------------------------------

# (none yet)

#-----------------------------------------------------------------
# End of GUI configuration file.  HelpMe continues to load the helpme
# startup configuration files.
#-----------------------------------------------------------------
